# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN
# Copyright (C) 2022 TU Wien.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Resolver and proxy for users."""

from types import SimpleNamespace

from flask_principal import RoleNeed, UserNeed
from invenio_access.permissions import system_process, system_user_id
from invenio_accounts.models import Role, User
from invenio_records_resources.references.entity_resolvers import (
    EntityProxy,
    EntityResolver,
)
from sqlalchemy.exc import NoResultFound

from .proxies import current_users_service
from .services.groups.config import GroupsServiceConfig
from .services.schemas import SystemUserSchema, UserGhostSchema
from .services.users.config import UsersServiceConfig


class UserProxy(EntityProxy):
    """Resolver proxy for a User entity."""

    def _resolve(self):
        """Resolve the User from the proxy's reference dict, or system_identity."""
        user_id = self._parse_ref_dict_id()
        if user_id == system_user_id:  # system_user_id is a string: "system"
            return self.system_record()
        else:
            try:
                return User.query.get(int(user_id))
            except NoResultFound:
                return self.ghost_record({"id": user_id})

    def get_needs(self, ctx=None):
        """Get the UserNeed for the referenced user."""
        user_id = self._parse_ref_dict_id()
        if user_id == system_user_id:
            return [system_process]
        else:
            return [UserNeed(int(user_id))]

    def ghost_record(self, value):
        """Return default representation of not resolved user."""
        return UserGhostSchema().dump(value)

    def system_record(self):
        """Return the representation of system user."""
        default_constant_values = {}
        return SystemUserSchema().dump(default_constant_values)

    def pick_resolved_fields(self, identity, resolved_dict):
        """Select which fields to return when resolving the reference."""
        profile = resolved_dict.get("profile", {})
        fake_user_obj = SimpleNamespace(id=resolved_dict["id"])
        avatar = current_users_service.links_item_tpl.expand(identity, fake_user_obj)[
            "avatar"
        ]
        # TODO trying to recreate the user item here, there must be a better way!
        links = resolved_dict.get("links", {})
        links["avatar"] = avatar
        serialized_user = {
            "id": resolved_dict["id"],
            "username": resolved_dict.get("username", ""),
            "email": resolved_dict.get("email", ""),
            "profile": {
                "full_name": profile.get("full_name", ""),
                "affiliations": profile.get("affiliations", ""),
            },
            "links": links,
            "blocked_at": resolved_dict.get("blocked_at"),
            "verified_at": resolved_dict.get("verified_at"),
            "confirmed_at": resolved_dict.get("confirmed_at"),
            "active": resolved_dict.get("active"),
            "is_current_user": resolved_dict.get("is_current_user"),
        }

        return serialized_user


class UserResolver(EntityResolver):
    """Resolver for users."""

    type_id = "user"

    def __init__(self):
        """Constructor."""
        # There's a bit of a mixup of type_key and type_id. Base resolver has no
        # type_key, but RecordResolvers have.
        self.type_key = self.type_id
        super().__init__(UsersServiceConfig.service_id)

    def matches_reference_dict(self, ref_dict):
        """Check if the reference dict references a user."""
        return self._parse_ref_dict_type(ref_dict) == self.type_id

    def _reference_entity(self, entity):
        """Create a reference dict for the given user."""
        return {"user": str(entity.id)}

    def matches_entity(self, entity):
        """Check if the entity is a user."""
        return isinstance(entity, User)

    def _get_entity_proxy(self, ref_dict):
        """Return a UserProxy for the given reference dict."""
        return UserProxy(self, ref_dict)


class GroupProxy(EntityProxy):
    """Resolver proxy for a Role entity."""

    def _resolve(self):
        """Resolve the User from the proxy's reference dict, or system_identity."""
        # Resolves to role name, not id
        role_id = self._parse_ref_dict_id()
        try:
            return Role.query.filter(
                Role.name == role_id  # TODO to be changed to role id
            ).one()
        except NoResultFound:
            return {}

    def pick_resolved_fields(self, identity, resolved_dict):
        """Select which fields to return when resolving the reference."""
        serialized_role = {
            "id": resolved_dict.get("id", ""),
            "name": resolved_dict.get("name", ""),
        }

        return serialized_role

    def get_needs(self, ctx=None):
        """Return needs based on the given roles."""
        role_id = self._parse_ref_dict_id()
        return [RoleNeed(role_id)]

    def ghost_record(self, value):
        """Return default representation of not resolved group.

        .. note::

            Only groups that are not indexed should need this. Non-indexed groups include groups that were not created by users
            e.g. user-moderation.
        """
        return {}


class GroupResolver(EntityResolver):
    """Group entity resolver."""

    type_id = "group"
    """Type identifier for this resolver."""

    def __init__(self):
        """Constructor."""
        # There's a bit of a mixup of type_key and type_id. Base resolver has no
        # type_key, but RecordResolvers have.
        self.type_key = self.type_id
        super().__init__(GroupsServiceConfig.service_id)

    def matches_reference_dict(self, ref_dict):
        """Check if the reference dict references a role."""
        return self._parse_ref_dict_type(ref_dict) == self.type_id

    def _reference_entity(self, entity):
        """Create a reference dict for the given user."""
        return {"group": str(entity.id)}

    def matches_entity(self, entity):
        """Check if the entity is a Role."""
        return isinstance(entity, Role)

    def _get_entity_proxy(self, ref_dict):
        """Return a GroupProxy for the given reference dict."""
        return GroupProxy(self, ref_dict)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 TU Wien.
# Copyright (C) 2022 European Union.
# Copyright (C) 2022-2024 CERN.
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Pytest configuration.

See https://pytest-invenio.readthedocs.io/ for documentation on which test
fixtures are available.
"""

import pytest
from flask_principal import AnonymousIdentity
from invenio_access.models import ActionRoles
from invenio_access.permissions import any_user as any_user_need
from invenio_access.permissions import system_identity
from invenio_accounts.models import Domain, DomainCategory, DomainOrg, Role
from invenio_accounts.proxies import current_datastore
from invenio_app.factory import create_api
from invenio_cache.proxies import current_cache
from marshmallow import fields

from invenio_users_resources.permissions import user_management_action
from invenio_users_resources.proxies import (
    current_domains_service,
    current_groups_service,
    current_users_service,
)
from invenio_users_resources.records import GroupAggregate
from invenio_users_resources.services.schemas import (
    NotificationPreferences,
    UserPreferencesSchema,
)

pytest_plugins = ("celery.contrib.pytest",)


class UserPreferencesNotificationsSchema(UserPreferencesSchema):
    """Schema extending preferences with notification preferences."""

    notifications = fields.Nested(NotificationPreferences)


#
# Application
#
@pytest.fixture(scope="module")
def app_config(app_config):
    """Override pytest-invenio app_config fixture."""
    app_config["RECORDS_REFRESOLVER_CLS"] = (
        "invenio_records.resolver.InvenioRefResolver"
    )
    app_config["RECORDS_REFRESOLVER_STORE"] = (
        "invenio_jsonschemas.proxies.current_refresolver_store"
    )
    # Variable not used. We set it to silent warnings
    app_config["JSONSCHEMAS_HOST"] = "not-used"
    # setting preferences schema to test notifications
    app_config["ACCOUNTS_USER_PREFERENCES_SCHEMA"] = UserPreferencesNotificationsSchema

    app_config["USERS_RESOURCES_GROUPS_ENABLED"] = True

    return app_config


@pytest.fixture(scope="module")
def create_app(instance_path):
    """Application factory fixture."""
    return create_api


@pytest.fixture()
def headers():
    """Default headers for making requests."""
    return {
        "content-type": "application/json",
        "accept": "application/json",
    }


#
# Services
#
@pytest.fixture(scope="module")
def user_service(app):
    """User service."""
    return current_users_service


@pytest.fixture(scope="module")
def group_service(app):
    """Group service."""
    return current_groups_service


#
# Users
#
@pytest.fixture(scope="module")
def anon_identity():
    """A new user."""
    identity = AnonymousIdentity()
    identity.provides.add(any_user_need)
    return identity


@pytest.fixture(scope="module")
def user_moderator(UserFixture, app, database, users):
    """Admin user for requests."""
    action_name = user_management_action.value
    moderator = users["user_moderator"]

    role = Role(name=action_name)
    database.session.add(role)

    action_role = ActionRoles.create(action=user_management_action, role=role)
    database.session.add(action_role)

    moderator.user.roles.append(role)
    database.session.commit()
    return moderator


@pytest.fixture(scope="module")
def users_data():
    """Data for users."""
    return [
        {
            "username": "pubres",
            "email": "pubres@inveniosoftware.org",
            "profile": {
                "full_name": "Tim Smith",
                "affiliations": "CERN",
            },
            "preferences": {
                "visibility": "public",
                "email_visibility": "restricted",
            },
        },
        {
            "username": "pub",
            "email": "pub@inveniosoftware.org",
            "profile": {
                "full_name": "Jose Benito Gonzalez Lopez",
                "affiliations": "CERN",
            },
            "preferences": {
                "visibility": "public",
                "email_visibility": "public",
            },
        },
        {
            "username": "res",
            "email": "res@inveniosoftware.org",
            "profile": {
                "full_name": "Donat Agosti",
                "affiliations": "Plazi",
            },
            "preferences": {
                "visibility": "restricted",
                "email_visibility": "restricted",
            },
        },
        {
            "username": "unconfirmed",
            "email": "unconfirmed@inveniosoftware.org",
            "confirmed": False,
        },
        {
            "username": "inactive",
            "email": "inactive@inveniosoftware.org",
            "profile": {
                "full_name": "Spammer",
                "affiliations": "Spam org",
            },
            "preferences": {
                "visibility": "public",
                "email_visibility": "public",
            },
            "active": False,
        },
        {
            "username": "notification_enabled",
            "email": "notification-enabled@inveniosoftware.org",
            "profile": {
                "full_name": "Mr. Worldwide",
                "affiliations": "World",
            },
            "preferences": {
                "visibility": "restricted",
                "email_visibility": "public",
                "notifications": {
                    "enabled": True,
                },
            },
        },
        {
            "username": "notification_disabled",
            "email": "notification-disabled@inveniosoftware.org",
            "profile": {
                "full_name": "Loner",
                "affiliations": "Home",
            },
            "preferences": {
                "visibility": "restricted",
                "email_visibility": "public",
                "notifications": {
                    "enabled": False,
                },
            },
        },
        {
            "username": "user_moderator",
            "email": "user_moderator@inveniosoftware.org",
            "profile": {
                "full_name": "Mr",
                "affiliations": "Admin",
            },
            "preferences": {
                "visibility": "restricted",
                "email_visibility": "public",
                "notifications": {
                    "enabled": False,
                },
            },
        },
    ]


@pytest.fixture(scope="module")
def users(UserFixture, app, database, users_data):
    """Test users."""
    users = {}
    for obj in users_data:
        u = UserFixture(
            username=obj["username"],
            email=obj["email"],
            password=obj["username"],
            user_profile=obj.get("profile"),
            preferences=obj.get("preferences"),
            active=obj.get("active", True),
            confirmed=obj.get("confirmed", True),
        )
        u.create(app, database)
        users[obj["username"]] = u
    current_users_service.indexer.process_bulk_queue()
    current_users_service.record_cls.index.refresh()
    database.session.commit()
    return users


def _create_group(id, name, description, is_managed, database):
    """Creates a Role/Group."""
    r = current_datastore.create_role(
        id=id, name=name, description=description, is_managed=is_managed
    )
    current_datastore.commit()
    return r


@pytest.fixture(scope="module")
def group(database):
    """A single group."""
    r = _create_group(
        id="it-dep",
        name="it-dep",
        description="IT Department",
        is_managed=True,
        database=database,
    )
    return r


@pytest.fixture(scope="module")
def group2(database):
    """A single group."""
    r = _create_group(
        id="hr-dep",
        name="hr-dep",
        description="HR Department",
        is_managed=True,
        database=database,
    )
    return r


@pytest.fixture(scope="module")
def not_managed_group(database):
    """An unmanaged group."""
    r = _create_group(
        id="not-managed-dep",
        name="not-managed-dep",
        description="A group which is not managed",
        is_managed=False,
        database=database,
    )
    return r


@pytest.fixture(scope="module")
def groups(database, group, group2, not_managed_group):
    """Available indexed groups."""
    roles = [group, group2, not_managed_group]

    current_groups_service.indexer.process_bulk_queue()
    current_groups_service.record_cls.index.refresh()
    return roles


@pytest.fixture(scope="module")
def user_pub(users):
    """User jbenito (restricted/restricted)."""
    return users["pub"]


@pytest.fixture(scope="module")
def user_pubres(users):
    """User tjs (public/restricted)."""
    return users["pubres"]


@pytest.fixture(scope="module")
def user_res(users):
    """User agosti (restricted/restricted)."""
    return users["res"]


@pytest.fixture(scope="module")
def user_inactive(users):
    """Inactive user."""
    return users["inactive"]


@pytest.fixture(scope="module")
def user_unconfirmed(users):
    """Unconfirmed user."""
    return users["unconfirmed"]


@pytest.fixture(scope="module")
def user_notification_enabled(users):
    """User with notfications enabled."""
    return users["notification_enabled"]


@pytest.fixture(scope="module")
def user_notification_disabled(users):
    """User with notfications disabled."""
    return users["notification_disabled"]


@pytest.fixture(scope="module")
def user_admin(users):
    """User with notfications disabled."""
    return users["admin_user"]


@pytest.fixture(scope="function")
def clear_cache():
    """Clear cache after each test in this module.

    Locking is done using cache, therefore the cache must be cleared after each test to make sure that locks from previous tests are cleared.
    """
    current_cache.cache.clear()


@pytest.fixture(scope="module")
def domains_data():
    """Data for domains."""
    return [
        {
            "domain": "cern.ch",
            "tld": "ch",
            "status": 3,
            "flagged": False,
            "flagged_source": "",
            "category": 1,
            "org_id": 1,
        },
        {
            "domain": "inveniosoftware.org",
            "tld": "org",
            "status": 3,
            "flagged": False,
            "flagged_source": "",
            "org_id": 2,
        },
        {
            "domain": "new.org",
            "tld": "org",
            "status": 1,
            "flagged": False,
            "flagged_source": "",
        },
        {
            "domain": "moderated.org",
            "tld": "org",
            "status": 2,
            "flagged": True,
            "flagged_source": "disposable",
            "category": 3,
        },
        {
            "domain": "spammer.com",
            "tld": "com",
            "status": 4,
            "flagged": True,
            "flagged_source": "",
            "category": 4,
        },
    ]


@pytest.fixture(scope="module")
def domaincategories_data():
    """Data for domains."""
    return [
        {"id": 1, "label": "organization"},
        {"id": 2, "label": "company"},
        {"id": 3, "label": "mail-provider"},
        {"id": 4, "label": "spammer"},
    ]


@pytest.fixture(scope="module")
def domainorgs_data():
    """Data for domains."""
    return [
        {
            "id": 1,
            "pid": "https://ror.org/01ggx4157",
            "name": "CERN",
            "json": {"country": "ch"},
        },
        {
            "id": 2,
            "pid": "https://ror.org/01ggx4157::it",
            "name": "IT department",
            "json": {"country": "ch"},
            "parent_id": 1,
        },
    ]


@pytest.fixture(scope="module")
def domains(app, database, domainorgs_data, domaincategories_data, domains_data):
    """Test domains."""
    for d in domaincategories_data:
        database.session.add(DomainCategory(**d))
    for d in domainorgs_data:
        database.session.add(DomainOrg(**d))
    database.session.commit()

    domains = {}
    for d in domains_data:
        database.session.add(Domain(**d))
        domains[d["domain"]] = d
    database.session.commit()

    current_domains_service.rebuild_index(system_identity)
    current_domains_service.indexer.process_bulk_queue()
    current_domains_service.record_cls.index.refresh()
    return domains



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
# Copyright (C) 2022 TU Wien.
# Copyright (C) 2023-2024 Graz University of Technology.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio module providing management APIs for users and roles/groups."""

from flask import Blueprint

blueprint = Blueprint(
    "invenio_users_resources",
    __name__,
    template_folder="templates",
    static_folder="static",
)


def create_users_resources_bp(app):
    """Create the users resource blueprint."""
    ext = app.extensions["invenio-users-resources"]
    return ext.users_resource.as_blueprint()


def create_groups_resources_bp(app):
    """Create the user groups resource blueprint."""
    ext = app.extensions["invenio-users-resources"]
    return ext.groups_resource.as_blueprint()


def create_domains_resources_bp(app):
    """Create the domains resource blueprint."""
    ext = app.extensions["invenio-users-resources"]
    return ext.domains_resource.as_blueprint()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 CERN.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Domains resource config."""

import marshmallow as ma
from flask_resources import (
    JSONDeserializer,
    JSONSerializer,
    RequestBodyParser,
    ResponseHandler,
)
from invenio_records_resources.resources import RecordResourceConfig


#
# Resource config
#
class DomainsResourceConfig(RecordResourceConfig):
    """User groups resource configuration."""

    blueprint_name = "domains"
    url_prefix = "/domains"

    # Request parsing
    request_headers = {}
    request_body_parsers = {
        "application/vnd.inveniordm.v1+json": RequestBodyParser(JSONDeserializer()),
        "application/json": RequestBodyParser(JSONDeserializer()),
    }
    default_content_type = "application/vnd.inveniordm.v1+json"

    # Response handling
    response_handlers = {
        "application/vnd.inveniordm.v1+json": ResponseHandler(JSONSerializer()),
        "application/json": ResponseHandler(JSONSerializer()),
    }
    default_accept_mimetype = "application/vnd.inveniordm.v1+json"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 CERN.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Domains search facets definitions."""

from invenio_i18n import gettext as _
from invenio_records_resources.services.records.facets import TermsFacet

status = TermsFacet(
    field="status_name",
    label=_("Status"),
    value_labels={
        "new": _("New"),
        "moderated": _("Moderated"),
        "verified": _("Verified"),
        "blocked": _("Blocked"),
    },
)


flagged = TermsFacet(
    field="flagged",
    label=_("Flagged"),
    value_labels={
        True: _("Yes"),
        False: _("No"),
    },
)


category = TermsFacet(
    field="category_name",
    label=_("Category"),
)


organisation = TermsFacet(
    field="org_names",
    label=_("Organisation"),
)


tld = TermsFacet(
    field="tld",
    label=_("Top-level domain"),
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
# Copyright (C) 2022 TU Wien.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio module providing management APIs for users and roles/groups."""

from invenio_i18n import lazy_gettext as _
from marshmallow import Schema, fields, validate

from invenio_users_resources.services.domains import facets as domainfacets
from invenio_users_resources.services.schemas import UserSchema
from invenio_users_resources.services.users import facets

USERS_RESOURCES_AVATAR_COLORS = [
    "#e06055",
    "#ff8a65",
    "#e91e63",
    "#f06292",
    "#673ab7",
    "#ba68c8",
    "#7986cb",
    "#3f51b5",
    "#5e97f6",
    "#00a4e4",
    "#4dd0e1",
    "#0097a7",
    "#d4e157",
    "#aed581",
    "#57bb8a",
    "#4db6ac",
    "#607d8b",
    "#795548",
    "#a1887f",
    "#fdd835",
    "#a3a3a3",
    "#556c60",
    "#605264",
    "#923035",
    "#915a30",
    "#55526f",
    "#67635a",
]

USERS_RESOURCES_SERVICE_SCHEMA = UserSchema
"""Schema used by the users service."""

USERS_RESOURCES_SEARCH = {
    "sort": ["bestmatch", "username", "email", "domain", "newest", "oldest", "updated"],
    "facets": ["status", "visibility", "domain_status", "domain", "affiliations"],
}
"""User search configuration."""

USERS_RESOURCES_SORT_OPTIONS = {
    "bestmatch": dict(
        title=_("Best match"),
        fields=["_score"],
    ),
    "username": dict(
        title=_("Username"),
        fields=["username", "-created"],
    ),
    "email": dict(
        title=_("Email"),
        fields=["email_hidden", "-created"],
    ),
    "domain": dict(
        title=_("Domain"),
        fields=["domain", "-created"],
    ),
    "newest": dict(
        title=_("Newest"),
        fields=["-created"],
    ),
    "oldest": dict(
        title=_("Oldest"),
        fields=["created"],
    ),
    "updated": dict(
        title=_("Recently updated"),
        fields=["-updated"],
    ),
}
"""Definitions of available Users sort options. """

USERS_RESOURCES_SEARCH_FACETS = {
    "domain": {
        "facet": facets.domain,
        "ui": {
            "field": "domain",
        },
    },
    "domain_status": {
        "facet": facets.domain_status,
        "ui": {
            "field": "domain_status",
        },
    },
    "affiliations": {
        "facet": facets.affiliations,
        "ui": {
            "field": "profile.affiliations.keyword",
        },
    },
    "status": {
        "facet": facets.status,
        "ui": {
            "field": "status",
        },
    },
    "visibility": {
        "facet": facets.visibility,
        "ui": {
            "field": "visibility",
        },
    },
}
"""Invenio requests facets."""

USERS_RESOURCES_ADMINISTRATION_ENABLED = True
"""Enable the user administration"""

USERS_RESOURCES_MODERATION_LOCK_DEFAULT_TIMEOUT = 30
"""Default timeout, in seconds, to lock a user when moderating."""

USERS_RESOURCES_MODERATION_LOCK_RENEWAL_TIMEOUT = 120
"""Renewal timeout, in seconds, to increase the lock time for a user when moderating."""


USERS_RESOURCES_DOMAINS_SEARCH = {
    "sort": [
        "bestmatch",
        "domain",
        "newest",
        "oldest",
        "updated",
        "num-users",
        "num-active",
        "num-inactive",
        "num-confirmed",
        "num-verified",
        "num-blocked",
    ],
    "facets": ["status", "flagged", "category", "organisation", "tld"],
}
"""User search configuration."""

USERS_RESOURCES_DOMAINS_SORT_OPTIONS = {
    "bestmatch": dict(
        title=_("Best match"),
        fields=["_score"],
    ),
    "domain": dict(
        title=_("Domain"),
        fields=["domain", "-created"],
    ),
    "newest": dict(
        title=_("Newest"),
        fields=["-created"],
    ),
    "oldest": dict(
        title=_("Oldest"),
        fields=["created"],
    ),
    "updated": dict(
        title=_("Recently updated"),
        fields=["-updated"],
    ),
    "num-users": dict(
        title=_("# Users"),
        fields=["-num_users"],
    ),
    "num-active": dict(
        title=_("# Active"),
        fields=["-num_active"],
    ),
    "num-inactive": dict(
        title=_("# Inactive"),
        fields=["-num_inactive"],
    ),
    "num-confirmed": dict(
        title=_("# Confirmed"),
        fields=["-num_confirmed"],
    ),
    "num-verified": dict(
        title=_("# Verified"),
        fields=["-num_verified"],
    ),
    "num-blocked": dict(
        title=_("# Blocked"),
        fields=["-num_blocked"],
    ),
}
"""Definitions of available Users sort options. """

USERS_RESOURCES_DOMAINS_SEARCH_FACETS = {
    "status": {
        "facet": domainfacets.status,
        "ui": {
            "field": "status",
        },
    },
    "flagged": {
        "facet": domainfacets.flagged,
        "ui": {
            "field": "flagged",
        },
    },
    "category": {
        "facet": domainfacets.category,
        "ui": {
            "field": "category",
        },
    },
    "organisation": {
        "facet": domainfacets.organisation,
        "ui": {
            "field": "organisation",
        },
    },
    "tld": {
        "facet": domainfacets.tld,
        "ui": {
            "field": "tld",
        },
    },
}
"""Invenio domains facets."""


class OrgPropsSchema(Schema):
    """Schema for validating domain org properties."""

    country = fields.String(validate=validate.Length(min=2, max=3))


USERS_RESOURCES_DOMAINS_ORG_SCHEMA = OrgPropsSchema
"""Domains organisation schema config."""

USERS_RESOURCES_GROUPS_ENABLED = True
"""Config to enable features related to existence of groups."""



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Notification related tests."""

from invenio_notifications.models import Notification, Recipient

from invenio_users_resources.notifications.filters import UserPreferencesRecipientFilter
from invenio_users_resources.notifications.generators import UserRecipient
from invenio_users_resources.records.api import UserAggregate
from invenio_users_resources.records.models import UserAggregateModel


def test_user_recipient_generator(
    user_notification_disabled, user_notification_enabled
):
    generator_disabled = UserRecipient(key="disabled")
    generator_enabled = UserRecipient(key="enabled")

    user_notifications_disabled = UserAggregate.from_model(
        user_notification_disabled.user
    ).dumps()
    user_notifications_enabled = UserAggregate.from_model(
        user_notification_enabled.user
    ).dumps()

    n = Notification(
        type="",
        context={
            "disabled": user_notifications_disabled,
            "enabled": user_notifications_enabled,
        },
    )

    recipients = {}

    generator_disabled(n, recipients=recipients)
    assert 1 == len(recipients)
    assert user_notifications_disabled["id"] in recipients.keys()

    generator_enabled(n, recipients=recipients)
    assert 2 == len(recipients)
    assert [
        user_notifications_disabled["id"],
        user_notifications_enabled["id"],
    ] == list(recipients)

    # checking that user does not get added twice
    generator_disabled(n, recipients=recipients)
    assert 2 == len(recipients)
    assert [
        user_notifications_disabled["id"],
        user_notifications_enabled["id"],
    ] == list(recipients)


def test_user_recipient_filter(user_notification_disabled, user_notification_enabled):
    """Test user recipient filter for notifications."""

    user_notifications_disabled = UserAggregate.from_model(
        user_notification_disabled.user
    ).dumps()
    user_notifications_enabled = UserAggregate.from_model(
        user_notification_enabled.user
    ).dumps()

    n = Notification(type="", context={})
    recipient_enabled = Recipient(data=user_notifications_enabled)
    recipient_disabled = Recipient(data=user_notifications_disabled)

    recipients = {
        user_notifications_disabled["id"]: recipient_disabled,
        user_notifications_enabled["id"]: recipient_enabled,
    }

    preferences_filter = UserPreferencesRecipientFilter()
    filtered_recipients = preferences_filter(
        notification=n,
        recipients=recipients,
    )

    assert 1 == len(filtered_recipients)
    assert recipient_enabled == filtered_recipients[user_notifications_enabled["id"]]
